@@ -71,7 +71,7 @@ def clear_managers():
     
 class Pool(object):
     def __init__(self, echo = False, use_threadlocal = True, logger=None, **kwargs):
-        self._threadconns = {} #weakref.WeakValueDictionary()
+        self._threadconns = weakref.WeakValueDictionary()
         self._use_threadlocal = use_threadlocal
         self.echo = echo
         self._logger = logger or util.Logger(origin='pool')
@@ -179,6 +179,8 @@ class SingletonThreadPool(Pool):
         self._conns = {}
         self._creator = creator
 
+    def dispose(self):
+        pass
     def status(self):
         return "SingletonThreadPool id:%d thread:%d size: %d" % (id(self), thread.get_ident(), len(self._conns))
 
