@@ -1,20 +1,16 @@
 import testbase
 import unittest
 
-testbase.echo = False
-
-#test
-
 def suite():
     modules_to_test = (
         # core utilities
-        'historyarray', 
+        'historyarray',
         'attributes', 
         'dependency',
-        
+
         # connectivity, execution
         'pool', 
-        'engine',
+        'transaction',
         
         # schema/tables
         'reflection', 
@@ -24,6 +20,7 @@ def suite():
         # SQL syntax
         'select',
         'selectable',
+        'case_statement', 
         
         # assorted round-trip tests
         'query',
@@ -39,8 +36,11 @@ def suite():
         'eagertest2',
         
         # ORM persistence
+        'sessioncontext', 
         'objectstore',
+        'cascade',
         'relationships',
+        'association',
         
         # cyclical ORM persistence
         'cycles',
@@ -50,9 +50,13 @@ def suite():
         'manytomany',
         'onetoone',
         'inheritance',
+        'polymorph',
         
         # extensions
         'proxy_engine',
+        'activemapper',
+        'sqlsoup'
+        
         #'wsgi_test',
         
         )
@@ -61,8 +65,6 @@ def suite():
         alltests.addTest(unittest.findTestCases(module, suiteClass=None))
     return alltests
 
-import sys
-sys.stdout = sys.stderr
 
 if __name__ == '__main__':
     testbase.runTests(suite())
