@@ -16,7 +16,7 @@ attr_manager = AttributeManager()
 if manage_attributes:
     attr_manager.register_attribute(User, 'id', uselist=False)
     attr_manager.register_attribute(User, 'name', uselist=False)
-    attr_manager.register_attribute(User, 'addresses', uselist=True)
+    attr_manager.register_attribute(User, 'addresses', uselist=True, trackparent=True)
     attr_manager.register_attribute(Address, 'email', uselist=False)
 
 now = time.time()
@@ -35,7 +35,7 @@ for i in range(0,130):
         a.email = 'foo@bar.com'
         u.addresses.append(a)
 #    gc.collect()
-    print len(managed_attributes)
+#    print len(managed_attributes)
 #    managed_attributes.clear()
 total = time.time() - now
 print "Total time", total
