@@ -17,10 +17,12 @@ class ParseConnectTest(PersistTest):
             'dbtype:///:memory:',
             'dbtype:///foo/bar/im/a/file',
             'dbtype:///E:/work/src/LEM/db/hello.db',
-            'dbtype://'
+            'dbtype://',
+            'dbtype://username:password@/db'
         ):
             u = url.make_url(text)
             print u, text
+            print "username=", u.username, "password=", u.password,  "database=", u.database, "host=", u.host
             assert str(u) == text
 
             
